@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2018 Michal Čihař <michal@cihar.com>
+# Copyright © 2012 - 2020 Michal Čihař <michal@cihar.com>
 #
 # This file is part of Weblate translation-finder
 # <https://github.com/WeblateOrg/translation-finder>
@@ -18,10 +18,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
-from __future__ import unicode_literals, absolute_import
+"""Translation finder, a module to locate translatable files in a filesystem."""
+from __future__ import absolute_import, unicode_literals
 
-from .finder import Finder
 from .api import discover
+from .discovery.result import DiscoveryResult
+from .finder import Finder
 
-__all__ = ("Finder", "discover")
-__version__ = "1.0"
+__all__ = ("Finder", "discover", "DiscoveryResult")
+__version__ = "1.8"
