@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2018 Michal Čihař <michal@cihar.com>
+# Copyright © 2012 - 2020 Michal Čihař <michal@cihar.com>
 #
 # This file is part of Weblate translation-finder
 # <https://github.com/WeblateOrg/translation-finder>
@@ -19,39 +19,53 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 """Base discovery code."""
-from __future__ import unicode_literals, absolute_import
+from __future__ import absolute_import, unicode_literals
 
-from itertools import chain
 import re
+from itertools import chain
 
 from chardet.universaldetector import UniversalDetector
 
+from ..countries import COUNTRIES
 from ..languages import LANGUAGES
+from .result import DiscoveryResult
 
-BLACKLIST = frozenset(("po", "ts"))
+TOKEN_SPLIT = re.compile(r"([_.-])")
 
-TOKEN_SPLIT = re.compile(r"([_-])")
+LOCALES = {"latn", "cyrl", "hant", "hans"}
 
 
 class BaseDiscovery(object):
     """Abstract base class for discovery."""
 
-    file_format = "auto"
-    file_format_mono = None
+    file_format = ""
     mask = "*.*"
     new_base_mask = None
+    origin = None
+    priority = 1000
 
     def __init__(self, finder, source_language="en"):
         self.finder = finder
         self.source_language = source_language
 
     @staticmethod
-    def is_language_code(code):
-        """Analysis whether passed parameter looks like language code."""
-        if code.lower() in BLACKLIST:
-            return False
+    def is_country_code(code):
+        code = code.lower()
+        return code in COUNTRIES or code in LOCALES
 
-        return code.lower().replace("-", "_") in LANGUAGES
+    @classmethod
+    def is_language_code(cls, code):
+        """Analysis whether passed parameter looks like language code."""
+        code = code.lower().replace("-", "_")
+        if code in LANGUAGES:
+            return True
+
+        if "_" in code:
+            lang, country = code.split("_", 1)
+            if lang in LANGUAGES and cls.is_country_code(country):
+                return True
+
+        return False
 
     def get_wildcard(self, part):
         """Generate language wilcard for a path part.
@@ -64,21 +78,34 @@ class BaseDiscovery(object):
             # Handle <language>.extension
             if self.is_language_code(base):
                 return "*.{}".format(ext)
+            # Handle prefix.<language>.extension
+            if "." in base:
+                prefix, token = base.split(".", 1)
+                if self.is_language_code(token):
+                    return "{}.*.{}".format(prefix, ext)
             # Handle prefix-<language>.extension or prefix_<language>.extension
             tokens = TOKEN_SPLIT.split(base)
             for pos, token in enumerate(tokens):
-                if token in ("-", "_"):
+                if token in ("-", "_", "."):
                     continue
                 if self.is_language_code(token):
-                    return "{}*.{}".format("".join(tokens[:pos]), ext)
+                    end = pos + 1
+                    if pos + 3 <= len(tokens):
+                        if self.is_language_code("".join(tokens[pos : pos + 3])):
+                            end = pos + 3
+                    return "{}*{}.{}".format(
+                        "".join(tokens[:pos]), "".join(tokens[end:]), ext
+                    )
         return None
 
     def fill_in_new_base(self, result):
         if not self.new_base_mask:
             return
         path = result["filemask"]
-        basename = path.rsplit("/", 1)[1].rsplit(".", 1)[0]
-        new_name = self.new_base_mask.replace("*", basename)
+        basename = path.rsplit("/", 1)[-1].rsplit(".", 1)[0]
+        if "*" in basename:
+            basename = basename.replace(".*", "").replace("-*", "").replace("*", "")
+        new_name = self.new_base_mask.replace("*", basename).lower()
 
         while "/" in path:
             path = path.rsplit("/", 1)[0]
@@ -89,21 +116,35 @@ class BaseDiscovery(object):
                 result["new_base"] = "/".join(match.parts)
                 return
 
-    def fill_in_template(self, result):
+    def has_storage(self, name):
+        """Check whether finder has a storage."""
+        return self.finder.has_file(name)
+
+    def get_language_aliases(self, language):
+        """Language code aliases"""
+        return [language]
+
+    def possible_templates(self, language, mask):
+        """Language code aliases"""
+        for alias in self.get_language_aliases(language):
+            yield mask.replace("*", alias)
+
+    def fill_in_template(self, result, source_language=None):
         if "template" not in result:
-            template = result["filemask"].replace("*", self.source_language)
-            if self.finder.has_file(template):
-                result["template"] = template
+            if source_language is None:
+                source_language = self.source_language
+            for template in self.possible_templates(
+                source_language, result["filemask"]
+            ):
+                if self.has_storage(template):
+                    result["template"] = template
+                    break
 
     def fill_in_file_format(self, result):
-        if "file_format" in result:
-            return
-        if "template" in result and self.file_format_mono:
-            result["file_format"] = self.file_format_mono
-        else:
+        if "file_format" not in result:
             result["file_format"] = self.file_format
 
-    def adjust_encoding(self, result):
+    def adjust_format(self, result):
         return
 
     def discover(self):
@@ -112,20 +153,28 @@ class BaseDiscovery(object):
         for result in self.get_masks():
             if result["filemask"] in discovered:
                 continue
-            if "template" in result and not self.finder.has_file(result["template"]):
+            if "template" in result and not self.has_storage(result["template"]):
                 continue
-            self.adjust_encoding(result)
             self.fill_in_template(result)
+            self.adjust_format(result)
             self.fill_in_new_base(result)
             self.fill_in_file_format(result)
             discovered.add(result["filemask"])
+            result = DiscoveryResult(result)
+            result.meta["discovery"] = self.__class__.__name__
+            result.meta["origin"] = self.origin
+            result.meta["priority"] = self.priority
             yield result
 
+    def filter_files(self):
+        """Filters possible file matches."""
+        return self.finder.filter_files(self.mask)
+
     def get_masks(self):
         """Return all file masks found in the directory.
 
         It is expected to contain duplicates."""
-        for path in self.finder.filter_files(self.mask):
+        for path in self.filter_files():
             parts = list(path.parts)
             skip = set()
             for pos, part in enumerate(parts):
@@ -134,10 +183,13 @@ class BaseDiscovery(object):
                 wildcard = self.get_wildcard(part)
                 if wildcard:
                     mask = parts[:]
-                    while part in mask:
-                        index = mask.index(part)
-                        skip.add(index)
-                        mask[index] = wildcard
+                    match = re.compile("(^|[._-]){}($|[._-])".format(re.escape(part)))
+                    for i, current in enumerate(mask):
+                        if match.findall(current):
+                            skip.add(i)
+                            mask[i] = match.sub(
+                                "\\g<1>{}\\g<2>".format(wildcard), current
+                            )
                     mask[pos] = wildcard
                     yield {"filemask": "/".join(mask)}
 
@@ -145,13 +197,13 @@ class BaseDiscovery(object):
 class EncodingDiscovery(BaseDiscovery):
     encoding_map = {}
 
-    def adjust_encoding(self, result):
+    def adjust_format(self, result):
         detector = UniversalDetector()
+        matches = [self.finder.mask_matches(result["filemask"])]
+        if "template" in result:
+            matches.append(self.finder.mask_matches(result["template"]))
 
-        for path in chain(
-            self.finder.mask_matches(result["filemask"]),
-            self.finder.mask_matches(result["template"]),
-        ):
+        for path in chain(*matches):
             if not hasattr(path, "open"):
                 continue
             with self.finder.open(path, "rb") as handle:
