@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2018 Michal Čihař <michal@cihar.com>
+# Copyright © 2012 - 2020 Michal Čihař <michal@cihar.com>
 #
 # This file is part of Weblate translation-finder
 # <https://github.com/WeblateOrg/translation-finder>
@@ -19,7 +19,7 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 """Filesystem finder for translations."""
-from __future__ import unicode_literals, absolute_import
+from __future__ import absolute_import, unicode_literals
 
 from fnmatch import fnmatch
 
@@ -28,55 +28,91 @@ try:
 except ImportError:
     from pathlib2 import Path, PurePath
 
-EXCLUDES = frozenset((".git", ".hg", ".eggs", "*.swp", "__pycache__"))
+EXCLUDES = {
+    ".git",
+    ".hg",
+    ".eggs",
+    "*.swp",
+    "__pycache__",
+    "__MACOSX",
+    "test",
+    "t",
+    ".deps",
+    "tests",
+    "test_data",
+    "testing",
+}
 
 
 class Finder(object):
     """Finder for files which might be considered translations."""
 
-    def __init__(self, root, files=None):
+    def __init__(self, root, mock=None):
         if not isinstance(root, PurePath):
             root = Path(root)
         self.root = root
-        if files is None:
-            files = self.list_files(root)
-        relatives = [(path, path.relative_to(root)) for path in files]
-        self.files = {path.as_posix(): path for absolute, path in relatives}
-        self.lc_files = {path.lower(): obj for path, obj in self.files.items()}
-        self.absolutes = {path.as_posix(): absolute for absolute, path in relatives}
+        if mock is None:
+            files = []
+            dirs = []
+            self.list_files(root, files, dirs)
+        else:
+            files, dirs = mock
+        # For the has_file/has_dir
+        self.filenames = {relative_path for absolute, relative, relative_path in files}
+        self.dirnames = {relative_path for absolute, relative, relative_path in dirs}
+        # Needed for filter_files
+        self.lc_files = [
+            (relative_path.lower(), relative)
+            for absolute, relative, relative_path in files
+        ]
+        self.lc_files.sort()
+        # Needed for open
+        self.absolutes = {
+            relative_path: absolute for absolute, relative, relative_path in files
+        }
+        # Needed for mask_matches
+        self.files = [
+            (relative_path, relative) for absolute, relative, relative_path in files
+        ]
+        self.files.sort()
 
-    def list_files(self, root):
-        """Recursively list files in a path.
+    def process_path(self, path):
+        relative = path.relative_to(self.root)
+        return (path, relative, relative.as_posix())
+
+    def list_files(self, root, files, dirs):
+        """Recursively list files and dirs in a path.
 
         It skips excluded files."""
+
         for path in root.iterdir():
             if any((path.match(exclude) for exclude in EXCLUDES)):
                 continue
+            if path.is_symlink():
+                continue
             if path.is_dir():
-                for ret in self.list_files(path):
-                    yield ret
+                dirs.append(self.process_path(path))
+                self.list_files(path, files, dirs)
             else:
-                yield path
+                files.append(self.process_path(path))
 
     def has_file(self, name):
         """Check whether file exists."""
-        return name in self.files
+        return name in self.filenames
+
+    def has_dir(self, name):
+        """Check whether dir exists."""
+        return name in self.dirnames
 
     def mask_matches(self, mask):
         """Return all mask matches."""
-        for name, path in sorted(self.files.items()):
+        for name, path in self.files:
             if fnmatch(name, mask):
                 yield path
 
     def filter_files(self, glob, dirglob=None):
         """Filter lowercase file names against glob."""
-        for name, path in sorted(self.lc_files.items()):
-            if (
-                "test" in path.parts
-                or "tests" in path.parts
-                or "test_data" in path.parts
-            ):
-                continue
+        for name, path in self.lc_files:
             try:
                 directory, filename = name.rsplit("/", 1)
             except ValueError:
